package com.seikoinstruments.sdk.mobileprinter.sample;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.Handler;
import android.support.v4.app.DialogFragment;
import android.view.View;
import android.view.WindowManager.LayoutParams;
import android.widget.Button;
import android.widget.LinearLayout;

import com.seikoinstruments.sdk.mobileprinter.CharacterSet;
import com.seikoinstruments.sdk.mobileprinter.ErrorCode;
import com.seikoinstruments.sdk.mobileprinter.InternationalCharacterSet;
import com.seikoinstruments.sdk.mobileprinter.PrinterException;
import com.seikoinstruments.sdk.mobileprinter.PrinterInterface;
import com.seikoinstruments.sdk.mobileprinter.PrinterManager;
import com.seikoinstruments.sdk.mobileprinter.PrinterModel;

import java.util.Locale;

public class CommandDialogFragment extends DialogFragment {

    public static final String KEY_PREF_SETTING = "preference_setting";
    public static final String KEY_SOCKET_KEEPING_TIME = "socket_keeping_time";
    public static final String KEY_WRITE_TIMEOUT = "write_timeout";
    public static final String KEY_RESPONSE_TIMEOUT = "response_timeout";
    public static final String KEY_BT_SECURE = "bt_secure";
    public static final String KEY_CHAR_SET = "character_set";
    public static final String KEY_INT_CHAR_SET = "int_character_set";
    protected static final int DEFAULT_SOCKET_KEEPING_TIME = 300;
    protected static final int DEFAULT_WRITE_TIMEOUT = 10000;
    protected static final int DEFAULT_RESPONSE_TIMEOUT = 10000;
    protected static final boolean DEFAULT_BT_SECURE = true;
    private static final int SOCKET_KEEPING_TIME_MIN = 60;
    private static final int SOCKET_KEEPING_TIME_MAX = 300;
    private static final int TIMEOUT_MIN = 1000;
    private static final int TIMEOUT_MAX = 90000;

    protected int mSocketKeepingTime = DEFAULT_SOCKET_KEEPING_TIME;
    protected boolean mBtSecure = DEFAULT_BT_SECURE;

    protected RequestSender mRequestSender;
    protected PrinterManager mPrinterManager;
    protected LinearLayout mLayout;
    protected ConnectionInformation mInfo;
    private Activity mActivity;
    private static boolean mDialogDisplaying = false;
    private boolean mIsDismiss = false;

    /* Override this method if necessary */
    protected void setLayout(View v) {
        this.mLayout = (LinearLayout) v;
    }

    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        mPrinterManager = MainActivity.sPrinterManager;
        mActivity = getActivity();
        setupWidgets();
        
        AlertDialog.Builder builder = new AlertDialog.Builder(mActivity);
        builder.setView(mLayout);
        Dialog dialog = builder.create();
        dialog.getWindow().setSoftInputMode(LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);

        mInfo = (ConnectionInformation) getArguments().getSerializable(MainActivity.CONN_INFO);
        return dialog;
    }

    @Override
    public void onDismiss(DialogInterface dialog) {
        mPrinterManager = null;
        super.onDismiss(dialog);
    }

    /* Override this method */
    protected void setupWidgets() {
        /* Set up widgets here */
    }

    protected void doRequest(final boolean isDismiss) {
        mIsDismiss = isDismiss;
        final MainActivity activity = (MainActivity) getActivity();
        activity.createProgressDialog(getString(R.string.dialog_message)).show();
        final PrinterInterface itf = getPrinterIf(mInfo);
        // Not allow to do network access on main thread
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    applySettingWithCurrentProperty();
                    mSocketKeepingTime = getPropertyInt(KEY_SOCKET_KEEPING_TIME);
                    mBtSecure = getPropertyBool(KEY_BT_SECURE);
                    mPrinterManager.open(itf, PrinterModel.PRN_MODEL_MP_A40,
                            mInfo.getAddress(), mSocketKeepingTime, mBtSecure);
                    popInfoDialog(mRequestSender.onRequestSend());
                } catch (PrinterException e) {
                    popPrinterExceptionDialog(e.getErrorCode());
                } finally {
                    if (mPrinterManager != null && mPrinterManager.isOpened()) {
                        try {
                            mPrinterManager.close();
                        } catch (PrinterException e) {
                            popPrinterExceptionDialog(e.getErrorCode());
                        }
                    }
                    activity.mHandler.post(new Runnable() {
                        @Override
                        public void run() {
                            activity.closeDialog();
                        }
                    });
                }
            }
        }, "Opening").start();
    }

    public interface RequestSender {
        public String onRequestSend() throws PrinterException;
    }

    public PrinterInterface getPrinterIf(ConnectionInformation info) {
        PrinterInterface titf = PrinterInterface.PRN_IF_BT;
        switch (info.getType()) {
            case 1:
                titf = PrinterInterface.PRN_IF_TCP;
                break;
            case 2:
                titf = PrinterInterface.PRN_IF_BT;
                break;
            case 3:
                titf = PrinterInterface.PRN_IF_USB;
                break;
        }
        return titf;
    }

    protected void applySetting(int writeTimeout, int responseTimeout, CharacterSet codepage,
                                InternationalCharacterSet intChar) throws PrinterException {
        mPrinterManager.setWriteTimeout(writeTimeout);
        mPrinterManager.setResponseTimeout(responseTimeout);
        mPrinterManager.selectCharacterSet(codepage);
        mPrinterManager.selectInternationalCharacterSet(intChar);
    }

    protected void applySettingWithCurrentProperty() throws PrinterException {
        int writeTimeout = getPropertyInt(KEY_WRITE_TIMEOUT);
        int responseTimeout = getPropertyInt(KEY_RESPONSE_TIMEOUT);
        CharacterSet codepage = getPropertyCharacterSet(KEY_CHAR_SET);
        InternationalCharacterSet intChar = getPropertyIntCharacterSet(KEY_INT_CHAR_SET);

        applySetting(writeTimeout, responseTimeout, codepage, intChar);
    }

    protected void setProperty(int socketKeepingTime, int writeTimeout,
                               int responseTimeout, boolean secure,
                               CharacterSet codepage, InternationalCharacterSet intChar) {
        socketKeepingTime = roundValue(KEY_SOCKET_KEEPING_TIME, socketKeepingTime);
        writeTimeout = roundValue(KEY_WRITE_TIMEOUT, writeTimeout);
        responseTimeout = roundValue(KEY_RESPONSE_TIMEOUT, responseTimeout);

        SharedPreferences pref = getSharedPreference();
        SharedPreferences.Editor editor = pref.edit();
        editor.clear();
        editor.putInt(KEY_SOCKET_KEEPING_TIME, socketKeepingTime);
        editor.putInt(KEY_WRITE_TIMEOUT, writeTimeout);
        editor.putInt(KEY_RESPONSE_TIMEOUT, responseTimeout);
        editor.putBoolean(KEY_BT_SECURE, secure);
        editor.putString(KEY_CHAR_SET, codepage.getCodePage());
        editor.putString(KEY_INT_CHAR_SET, intChar.toString());
        editor.commit();
    }

    protected void setLocaleProperty(CharacterSet codepage, InternationalCharacterSet intChar) {
        SharedPreferences pref = getSharedPreference();
        SharedPreferences.Editor editor = pref.edit();
        editor.remove(KEY_CHAR_SET);
        editor.remove(KEY_INT_CHAR_SET);
        editor.putString(KEY_CHAR_SET, codepage.getCodePage());
        editor.putString(KEY_INT_CHAR_SET, intChar.toString());
        editor.commit();
    }

    private int roundValue(String key, int value) {
        int ret = value;
        switch (key) {
            case KEY_SOCKET_KEEPING_TIME:
                if (value < SOCKET_KEEPING_TIME_MIN) {
                    ret = SOCKET_KEEPING_TIME_MIN;
                }
                if (SOCKET_KEEPING_TIME_MAX < value) {
                    ret = SOCKET_KEEPING_TIME_MAX;
                }
                break;
            case KEY_WRITE_TIMEOUT:
            case KEY_RESPONSE_TIMEOUT:
                if (value < TIMEOUT_MIN) {
                    ret = TIMEOUT_MIN;
                }
                if (TIMEOUT_MAX < value) {
                    ret = TIMEOUT_MAX;
                }
                break;
            default:
                break;
        }
        return ret;
    }

    protected int getPropertyInt(String key) {
        SharedPreferences pref = getSharedPreference();

        int defaultValue = 0;
        switch (key) {
            case KEY_SOCKET_KEEPING_TIME:
                defaultValue = DEFAULT_SOCKET_KEEPING_TIME;
                break;
            case KEY_WRITE_TIMEOUT:
                defaultValue = DEFAULT_WRITE_TIMEOUT;
                break;
            case KEY_RESPONSE_TIMEOUT:
                defaultValue = DEFAULT_RESPONSE_TIMEOUT;
                break;
            default:
                break;
        }

        return pref.getInt(key, defaultValue);
    }

    protected boolean getPropertyBool(String key) {
        SharedPreferences pref = getSharedPreference();

        return pref.getBoolean(key, DEFAULT_BT_SECURE);
    }

    protected CharacterSet getPropertyCharacterSet(String key) {
        SharedPreferences pref = getSharedPreference();

        CharacterSet defaultValue = mPrinterManager.getCharacterSet();
        String charSetName = pref.getString(key, defaultValue.toString());

        for (CharacterSet cs : CharacterSet.values()) {
            if (charSetName.equals(cs.getCodePage())) {
                return cs;
            }
        }

        return defaultValue;
    }

    protected InternationalCharacterSet getPropertyIntCharacterSet(String key) {
        SharedPreferences pref = getSharedPreference();

        InternationalCharacterSet defaultValue = mPrinterManager.getInternationalCharacterSet();
        String intCharSetName = pref.getString(key, defaultValue.toString());

        for (InternationalCharacterSet ics : InternationalCharacterSet.values()) {
            if (intCharSetName.equals(ics.toString())) {
                return ics;
            }
        }

        return defaultValue;
    }

    public void setActivity(Activity activity) {
        mActivity = activity;
    }

    private SharedPreferences getSharedPreference() {
        return mActivity.getSharedPreferences(KEY_PREF_SETTING, Activity.MODE_PRIVATE);
    }

    public void popInfoDialog(String msg) {
        String title = mActivity.getString(R.string.printer_info_title);
        popDialog(title, msg);
    }

    public void popPrinterExceptionDialog(ErrorCode errorCode) {
        String title = mActivity.getString(R.string.printer_exception_title);
        popDialog(title, mActivity.getString(R.string.err_code_msg) + errorCode);
    }

    public void popDialog(final String title, final String msg) {
        if (!mDialogDisplaying) {
            mDialogDisplaying = true;
            mActivity.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    AlertDialog.Builder builder = new AlertDialog.Builder(mActivity);
                    builder.setTitle(title);
                    builder.setMessage(msg);
                    builder.setPositiveButton(mActivity.getString(R.string.ok_button),
                            new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialogInterface, int i) {
                            mDialogDisplaying = false;
                            if (mIsDismiss) {dismiss();}
                        }
                    });
                    builder.setOnCancelListener(new DialogInterface.OnCancelListener() {
                        @Override
                        public void onCancel(DialogInterface dialogInterface) {
                            mDialogDisplaying = false;
                        }
                    });
                    AlertDialog dialog = builder.create();
                    dialog.show();
                }
            });
        }
    }

}

