package com.seikoinstruments.sdk.mobileprinter.sample;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.support.v4.app.DialogFragment;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.RelativeLayout;

import com.seikoinstruments.sdk.mobileprinter.DiscoveredPrinter;
import com.seikoinstruments.sdk.mobileprinter.PrinterDiscovery;
import com.seikoinstruments.sdk.mobileprinter.PrinterException;
import com.seikoinstruments.sdk.mobileprinter.PrinterInterface;
import com.seikoinstruments.sdk.mobileprinter.PrinterListener;

import java.util.List;

/**
 * Scanner activity
 */
public class ScannerDialogFragment extends DialogFragment {

    private static final String TITLE = "title";
    public static final int REQUEST_UPDATE_ADDRESS = 0x01;
    public static final int REQUEST_DISCOVERY_COMPLETE = 0x02;
    public static final int ERROR_FAIL_MESSAGE = 0x03;

    private ListView mListViewAddress;
    private ProgressDialog mDialog;

    private PrinterDiscovery mPrinterDiscovery;
    private ConnectionInformation mInfo;
    private List<DiscoveredPrinter> mAddressList;
    private ArrayAdapter<String> mAdapterAddress;
    private boolean mScanning = false;
    private boolean mFirstCall = false;

    // Use this instance of the interface to deliver action events
    // so that we can pass event back to the dialog's host
    private NoticeDialogListener mNoticeDialogListener;

    private AdapterView.OnItemClickListener mAddressListener =
            new AdapterView.OnItemClickListener() {
        @Override
        public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
           PrinterInterface itf = getPrinterIf();
            String address = "";
            switch (itf) {
                case PRN_IF_TCP:
                    address = mAddressList.get(position).getIpAddress();
                    break;
                case PRN_IF_BT:
                    address = mAddressList.get(position).getBluetoothAddress();
                    break;
                default:
                    break;
            }
            mNoticeDialogListener.onAddressClick(address);
            dismiss();
        }
    };

    // Handler that update list content
    private Handler mHandler = new Handler(Looper.getMainLooper()) {
        @Override
        public void handleMessage(Message message) {
            switch (message.what) {
                case REQUEST_UPDATE_ADDRESS:
                    String address = (String) message.obj;
                    mAdapterAddress.add(address);
                    break;
                case ERROR_FAIL_MESSAGE:
                    PrinterException e = (PrinterException) message.obj;
                    popDialog(e.getClass().getSimpleName(),
                            getString(R.string.err_code_msg) + e.getErrorCode());
                    break;
                case REQUEST_DISCOVERY_COMPLETE:
                    dismissProgressDialog();
                    break;
            }
        }
    };

    public static ScannerDialogFragment newInstance(String title, ConnectionInformation info) {
        ScannerDialogFragment scannerDialogFragment = new ScannerDialogFragment();
        Bundle args = new Bundle();
        args.putString(TITLE, title);
        args.putSerializable(MainActivity.CONN_INFO, info);
        scannerDialogFragment.setArguments(args);
        return scannerDialogFragment;
    }

    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        String title = getArguments().getString(TITLE);

        mInfo = (ConnectionInformation) getArguments().getSerializable(MainActivity.CONN_INFO);
        mFirstCall = true;

        // Use the Builder class for convenient dialog construction
        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
        RelativeLayout scannerDialogLayout = (RelativeLayout) getActivity().getLayoutInflater().
                inflate(R.layout.scanner_dialog, null);
        // Set up widgets
        setupWidgets(scannerDialogLayout);
        // Set up dialog layout
        builder.setView(scannerDialogLayout)
                .setTitle(title);

        // Create the AlertDialog and return it
        return builder.create();
    }

    @Override
    public void onResume() {
        super.onResume();
        if (mFirstCall) {
            onScanStart();
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        if(mScanning) {
            onScanCancel();
        }
    }

    // Override the Fragment.onAttach() method to instantiate the NoticeDialogListner
    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
        // Verify if the host activity implements the callback interface
        try {
            // Instantiate the NoticeDialogListener so we can send events to the host
            mNoticeDialogListener = (NoticeDialogListener) activity;
        } catch (ClassCastException e) {
            throw new ClassCastException(activity.toString()
                    + " must implement NoticeDialogListener.");
        }
    }

    private void setupWidgets(View parent) {
        // Set up address list
        mListViewAddress = (ListView) parent.findViewById(R.id.address_list);
        mAdapterAddress = new ArrayAdapter<String>(getActivity(), R.layout.address);
        mListViewAddress.setAdapter(mAdapterAddress);

        // Set up buttons
        Button btnScan = (Button) parent.findViewById(R.id.btn_scan);
        btnScan.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                onScanStart();
            }
        });
    }

    public interface NoticeDialogListener {
        public void onAddressClick(String address);
    }

    private void onScanStart() {
        if( mScanning ) {
            return;
        }

        final PrinterInterface itf = getPrinterIf();

        // Flush listview
        mAdapterAddress.clear();

        // Set up listview item listener
        mListViewAddress.setOnItemClickListener(mAddressListener);

        // Discvoery printer on another thread
        mPrinterDiscovery = new PrinterDiscovery(getActivity());
        Thread t = new Thread(new DiscoveryRunnable(mPrinterDiscovery, new PrinterListener() {
            @Override
            public void onDiscoveryFinished(int event) {
                if (event == PrinterListener.EVENT_FINISHED_DISCOVERY
                        || event == PrinterListener.EVENT_CANCELLED_DISCOVERY) {
                    try {
                        switch (itf) {
                            case PRN_IF_TCP:
                                listTcpFoundPrinter();
                                break;
                            case PRN_IF_BT:
                                listBluetoothFoundPrinter();
                                break;
                            default:
                                break;
                        }
                    } catch (PrinterException e) {
                        popDialog(e.getClass().getSimpleName(),
                                getString(R.string.err_code_msg) + e.getErrorCode());
                    } finally {
                        mScanning = false;
                    }
                }
            }
        }, itf), "discovery thread");
        t.start();
        mFirstCall = false;
    }

    private PrinterInterface getPrinterIf() {
        return getFragment().getPrinterIf(mInfo);
    }

    private CommandDialogFragment getFragment() {
        CommandDialogFragment fm = new CommandDialogFragment();
        fm.setActivity(getActivity());
        return fm;
    }

    private void showProgressDialog() {
        final PrinterInterface itf = getPrinterIf();
        String title = "";
        switch (itf) {
            case PRN_IF_TCP:
                title = getString(R.string.device_tcp);
                break;
            case PRN_IF_BT:
                title = getString(R.string.device_bt);
                break;
            default:
                break;
        }

        mDialog = new ProgressDialog(getActivity());
        mDialog.setTitle(title);
        mDialog.setMessage(getString(R.string.scanning_msg));
        mDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        mDialog.setCanceledOnTouchOutside(true);
        mDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
            @Override
            public void onCancel(DialogInterface dialogInterface) {
                //cancel scan
                // 1. when back key pressed
                // 2. when outside touched
                onScanCancel();
            }
        });
        mDialog.show();
    }

    private void dismissProgressDialog() {
        if (mDialog != null) {
            mDialog.dismiss();
        }
    }

    private void onScanCancel() {
        try {
            mPrinterDiscovery.cancelDiscoveryPrinter();
        } catch (PrinterException e) {
            mHandler.obtainMessage(ERROR_FAIL_MESSAGE, e).sendToTarget();
        }
    }

    private void listTcpFoundPrinter() throws PrinterException {
        mAddressList = mPrinterDiscovery.getFoundPrinter();
        for (DiscoveredPrinter printer : mAddressList) {
            String msg = printer.getPrinterModel() + "\n" + printer.getIpAddress();
            mHandler.obtainMessage(REQUEST_UPDATE_ADDRESS, msg)
                    .sendToTarget();
        }
        mHandler.obtainMessage(REQUEST_DISCOVERY_COMPLETE).sendToTarget();
    }

    private void listBluetoothFoundPrinter() throws PrinterException {
        mAddressList = mPrinterDiscovery.getFoundPrinter();
        for (DiscoveredPrinter printer : mAddressList) {
            String msg = printer.getPrinterModel() + "\n" + printer.getBluetoothAddress();
            mHandler.obtainMessage(REQUEST_UPDATE_ADDRESS, msg)
                    .sendToTarget();
        }
        mHandler.obtainMessage(REQUEST_DISCOVERY_COMPLETE).sendToTarget();
    }

    public void popDialog(final String title, final String msg) {
        getFragment().popDialog(title, msg);
    }

    private class DiscoveryRunnable implements Runnable {

        PrinterDiscovery mDiscovery;
        PrinterListener mPrinterListener;
        PrinterInterface mPrinterInterface;

        public DiscoveryRunnable(
                PrinterDiscovery pd, PrinterListener pl, PrinterInterface pi) {
            mDiscovery = pd;
            mPrinterListener = pl;
            mPrinterInterface = pi;
        }

        @Override
        public void run() {
            Looper.prepare();
            try {
                mDiscovery.startDiscoveryPrinter(
                        mPrinterListener, mPrinterInterface, 1, 5);
                mScanning = true;
                showProgressDialog();
            } catch (PrinterException e) {
                mHandler.obtainMessage(ERROR_FAIL_MESSAGE, e).sendToTarget();
            }
            Looper.loop();
        }
    }
}
